package com.openwide.android;

import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.PendingIntent;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.telephony.PhoneNumberUtils;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.inputmethod.EditorInfo;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.TextView;

public class MultiSmsSender extends Activity {
	private Button mAddButton;
	private Button mSend;
	private TextView mContacts;
	private TextView mEditor;
	private CheckBox mDeliveryCheckBox;

	private DeliveryDbAdapter mDbHelper;
	private ProgressDialog mSendingDialog;

	public static final int ACTIVITY_EDIT = 0;
	public static final int ACTIVITY_ADD_GROUP = 1;
	public static final int ACTIVITY_DELIVERY = 2;
	private static final int SENDING_DIALOG_KEY = 0;
	private static final int INSERT_ID = Menu.FIRST;

	public static final String PARAM_NUMBERS_LIST = "param number list";
	public static final String PARAM_FLUSH = "param flush";
	public static final String PARAM_ENTRY_ID = "entry_id";
	
	public static final String DEBUG_TAG="MultiSmsSender";

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);

		mDbHelper = new DeliveryDbAdapter(this);
		mDbHelper.open();

		mAddButton = (Button) findViewById(R.id.contacts);
		mSend = (Button) findViewById(R.id.send);
		mContacts = (TextView) findViewById(R.id.numbers);
		mEditor = (TextView) findViewById(R.id.editor);
		mDeliveryCheckBox = (CheckBox) findViewById(R.id.deliveryCheckBox);

		mContacts.setImeOptions(EditorInfo.IME_ACTION_NEXT);
		mEditor.setImeOptions(EditorInfo.IME_ACTION_DONE);

		mAddButton.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				selectNumbers();
			}
		});

		mSend.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				sendMessage();

			}
		});
	}

	public void selectNumbers() {
		Intent i = new Intent(this, PhoneNumberSelection.class);
		String rawNumbers = mContacts.getText().toString();
		String[] numbers = rawNumbers.split(",");
		i.putExtra(PARAM_NUMBERS_LIST, numbers);
		startActivityForResult(i, ACTIVITY_EDIT);
	}

	public void sendMessage() {
		MySMSManager manager = MySMSManager.getInstance();
		String message = mEditor.getText().toString();
		
		if("".equals(message)) {
			return;
		}
		
		String[] numbers = mContacts.getText().toString().split(",");
		HashSet<String> allreadySend = new HashSet<String>();
		int size = numbers.length;
		boolean haveDeliveryReports = mDeliveryCheckBox.isChecked();
		long deliveryId = -1;
		ArrayList<String> messages = manager.divideMessage(message);
		int messageCount = messages.size();

		//showDialog(SENDING_DIALOG_KEY);
		mSendingDialog = new ProgressDialog(this);
		mSendingDialog.setTitle(R.string.sending);
		mSendingDialog.setMessage(getResources().getString(R.string.wait));
		mSendingDialog.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
		mSendingDialog.setIndeterminate(true);
		mSendingDialog.setCancelable(false);
		mSendingDialog.setMax(size);
		mSendingDialog.show();

		if (haveDeliveryReports) {
			deliveryId = mDbHelper.createDelivery(message.substring(0, Math.min(30, message
					.length())).replace('\n', ' '), DateFormat.getDateInstance()
					.format(new Date()));

		}
		if (numbers.length != 0) {
			for (int i = 0; i < numbers.length; i++) {
				String newN = numbers[i].trim();

				if (!newN.equals("")
						&& PhoneNumberUtils.isWellFormedSmsAddress(newN)) {
					mSendingDialog.setProgress(i / size);

					if (!allreadySend.contains(newN)) {
						allreadySend.add(newN);

						ArrayList<PendingIntent> deliveryIntents = new ArrayList<PendingIntent>(messageCount);
						ArrayList<PendingIntent> sentIntents = null;

						

						if (haveDeliveryReports) {
							long entryId = mDbHelper.createEntry(mDbHelper.nameFromNumber(newN), newN, deliveryId);
							Log.d(DEBUG_TAG, "entry is "+entryId);
							for (int j = 0; j < messageCount; j++) {
								Uri entryURI = Uri.withAppendedPath(EntryContentProvider.CONTENT_URI, ""+entryId);
								Intent intent = new Intent(MessageReceiver.MESSAGE_RECEIVED,entryURI, this, MessageReceiver.class);
								//intent.putExtra(PARAM_ENTRY_ID, entryId);
								deliveryIntents.add(PendingIntent.getBroadcast(
										this, 0, intent, 0));
							}


						}
						manager.sendMultipartTextMessage(newN, null, messages, sentIntents, deliveryIntents );

					}
				}
			}
			mSendingDialog.dismiss();
			new AlertDialog.Builder(this).setPositiveButton(
					getResources().getString(R.string.ok),
					new DialogInterface.OnClickListener() {

						@Override
						public void onClick(DialogInterface dialog, int which) {
							dialog.dismiss();

						}
					}).setMessage(
							allreadySend.size() + " "
							+ getResources().getString(R.string.message_sent))
							.show();
		} else {
			new AlertDialog.Builder(this).setPositiveButton(
					getResources().getString(R.string.ok),
					new DialogInterface.OnClickListener() {

						@Override
						public void onClick(DialogInterface dialog, int which) {
							dialog.dismiss();

						}
					}).setMessage(
							getResources().getString(R.string.enter_number)).show();
		}

	}

	@Override
	protected Dialog onCreateDialog(int id) {
		switch (id) {
		case SENDING_DIALOG_KEY: {
			mSendingDialog = new ProgressDialog(this);
			mSendingDialog.setTitle(R.string.sending);
			mSendingDialog.setMessage(getResources().getString(R.string.wait));
			mSendingDialog.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
			mSendingDialog.setIndeterminate(true);
			mSendingDialog.setCancelable(false);

			return mSendingDialog;
		}
		}
		return null;
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode,
			Intent intent) {
		super.onActivityResult(requestCode, resultCode, intent);
		switch (requestCode) {
		case ACTIVITY_ADD_GROUP:
		case ACTIVITY_EDIT:
			if (intent != null) {
				String[] numbers = intent.getExtras().getStringArray(
						PARAM_NUMBERS_LIST);
				boolean flush = intent.getExtras().getBoolean(PARAM_FLUSH);
				String string = "";
				HashSet<String> res = new HashSet<String>();
				for (int i = 0; i < numbers.length; i++) {
					String newN = numbers[i].trim();
					if (!newN.equals("")) {
						res.add(newN);
					}
				}
				if (!flush) {
					String oldContactsString = mContacts.getText().toString();
					String[] oldContacts = oldContactsString.split(",");
					for (int i = 0; i < oldContacts.length; i++) {
						String newN = oldContacts[i].trim();
						if (!newN.equals("")) {
							res.add(newN);
						}
					}
				}

				for (String number : res) {
					string += number + ", ";
				}

				mContacts.setText(string);
			}
			break;

		default:
			break;
		}

	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		super.onCreateOptionsMenu(menu);
		menu.add(0, INSERT_ID, 0, R.string.add_group);
		menu.add(0, INSERT_ID + 1, 0, R.string.delivery);
		return true;
	}

	@Override
	public boolean onMenuItemSelected(int featureId, MenuItem item) {
		switch (item.getItemId()) {
		case INSERT_ID:
			display_group_list();
			return true;
		case INSERT_ID + 1:
			display_delivery_list();
			return true;
		}
		return super.onMenuItemSelected(featureId, item);
	}

	protected void onDestroy() {
		mDbHelper.close();
		super.onDestroy();
	}

	public void display_group_list() {

		Intent i = new Intent(this, SelectGroupActivity.class);
		startActivityForResult(i, ACTIVITY_ADD_GROUP);
	}

	public void display_delivery_list() {

		Intent i = new Intent(this, SelectDeliveryActivity.class);
		startActivityForResult(i, ACTIVITY_DELIVERY);
	}



}